@@ -1,36 +1,43 @@
 package DTL::Fast;
 use strict; use warnings FATAL => 'all'; 
-use parent 'Exporter';
-use Carp;
+use Exporter 'import';
 use Digest::MD5 qw(md5_hex);
 
 use 5.010;
-our $VERSION = 'v1.05.6';
+our $VERSION = '1.609.0_1'; # ==> ALSO update the version in the pod text below!
+
+# loaded modules
+our %TAG_HANDLERS;
+our %FILTER_HANDLERS;
+our %OPS_HANDLERS;
+
+# known but not loaded modules
+our %KNOWN_TAGS;        # plain map tag => module
+our %KNOWN_FILTERS;     # plain map filter => module
+our %KNOWN_OPS;         # complex map priority => operator => module
+our %KNOWN_OPS_PLAIN;   # plain map operator => module
+our @OPS_RE = ();
+
+# modules hash to avoid duplicating on deserializing
+our %LOADED_MODULES;
 
 require XSLoader;
 XSLoader::load('DTL::Fast', $VERSION);
 
-use DTL::Fast::Template;
-use DTL::Fast::Cache::Runtime;
-use DTL::Fast::Cache::Serialized;
-
 our $RUNTIME_CACHE;
 our $SERIALIZED_CACHE;
 
 our @EXPORT_OK;
 
-# @todo Texts should be tossed by references
-# @todo These should be done via Cache class which can work with last modified date
-
 push @EXPORT_OK, 'get_template';
 sub get_template
 {
     my( $template_name, %kwargs ) = @_;
     
-    croak  "Template name was not specified" 
+    die  "Template name was not specified" 
         if not $template_name;
 
-    croak "Template directories array was not specified"
+    die "Template directories array was not specified"
         if
             not defined $kwargs{'dirs'}
             or ref $kwargs{'dirs'} ne 'ARRAY'
@@ -56,7 +63,7 @@ sub get_template
         }
         else
         {
-            croak  sprintf( <<'_EOT_', $template_name, join("\n", @{$kwargs{'dirs'}}));
+            die  sprintf( <<'_EOT_', $template_name, join("\n", @{$kwargs{'dirs'}}));
 Unable to find template %s in directories: 
 %s
 _EOT_
@@ -89,7 +96,7 @@ sub read_template
     my $template = undef;
     my $template_path = undef;
 
-    croak "Template directories array was not specified"
+    die "Template directories array was not specified"
         if not defined $kwargs{'dirs'}
             or not ref $kwargs{'dirs'}
             or not scalar @{$kwargs{'dirs'}}
@@ -159,34 +166,44 @@ sub _read_file
             and -r $template_path
         )
         {
-            if( open my $IF, '<', $template_path )
-            {
-                local $/ = undef;
-                $template = <$IF>;
-                close $IF;
-                last;
-            }
-            else
-            {
-                croak  sprintf(
-                    'Error opening file %s, %s'
-                    , $template_path
-                    , $!
-                );
-            }
+            $template = __read_file( $template_path );
+            last;
         }        
     }
     
     return ($template, $template_path);
 }
 
+
+sub __read_file
+{
+    my( $file_name ) = @_;
+    my $result;
+    
+    if( open my $IF, '<', $file_name )
+    {
+        local $/ = undef;
+        $result = <$IF>;
+        close $IF;
+    }
+    else
+    {
+        die  sprintf(
+            'Error opening file %s, %s'
+            , $file_name
+            , $!
+        );
+    }
+    return $result;
+}
+
 # result should be cached with full list of params
 push @EXPORT_OK, 'select_template';
 sub select_template
 {
     my( $template_names, %kwargs ) = @_;
     
-    croak  "First parameter must be a template names array reference" 
+    die  "First parameter must be a template names array reference" 
         if(
             not ref $template_names
             or ref $template_names ne 'ARRAY'
@@ -206,6 +223,119 @@ sub select_template
     return $result;
 }
 
+# registering tag as known
+push @EXPORT_OK, 'register_tag';
+sub register_tag
+{
+    my( %tags ) = @_;
+    
+    while( my( $slug, $module) = each %tags )
+    {
+        $DTL::Fast::KNOWN_TAGS{lc($slug)} = $module;
+    }
+    
+    return;
+}
+
+# registering tag as known
+push @EXPORT_OK, 'preload_tags';
+sub preload_tags
+{
+    require Module::Load;
+    
+    while( my( $keyword, $module) = each %KNOWN_TAGS )
+    {
+        Module::Load::load($module);
+        $LOADED_MODULES{$module} = time;
+        delete $TAG_HANDLERS{$keyword} if exists $TAG_HANDLERS{$keyword} and $TAG_HANDLERS{$keyword} ne $module;
+    }
+    
+    return 1;
+}
+
+
+# registering filter as known
+push @EXPORT_OK, 'register_filter';
+sub register_filter
+{
+    my( %filters ) = @_;
+    
+    while( my( $slug, $module) = each %filters )
+    {
+        $DTL::Fast::KNOWN_FILTERS{$slug} = $module;
+        delete $FILTER_HANDLERS{$slug} if exists $FILTER_HANDLERS{$slug} and $FILTER_HANDLERS{$slug} ne $module;
+    }
+    
+    return;
+}
+
+push @EXPORT_OK, 'preload_filters';
+sub preload_filters
+{
+    require Module::Load;
+    
+    while( my( $keyword, $module) = each %KNOWN_FILTERS )
+    {
+        Module::Load::load($module);
+        $LOADED_MODULES{$module} = time;
+    }
+    
+    return 1;
+}
+
+# invoke with parameters:
+#
+#   '=' => [ priority, module ]
+#
+push @EXPORT_OK, 'register_operator';
+sub register_operator
+{
+    my %ops = @_;
+    
+    my %recompile = ();
+    foreach my $operator (keys %ops)
+    {
+        my($priority, $module) = @{$ops{$operator}};
+        
+        die "Operator priority must be a number from 0 to 8"
+            if $priority !~ /^[012345678]$/;
+        
+        $KNOWN_OPS{$priority} //= {};
+        $KNOWN_OPS{$priority}->{$operator} = $module;
+        $recompile{$priority} = 1;
+        $KNOWN_OPS_PLAIN{$operator} = $module;
+        delete $OPS_HANDLERS{$operator} if exists $OPS_HANDLERS{$operator} and $OPS_HANDLERS{$operator} ne $module;
+    }
+    
+    foreach my $priority (keys(%recompile))
+    {
+        my @ops = sort{ length $b <=> length $a } keys(%{$KNOWN_OPS{$priority}});
+        my $ops = join '|', map{ "\Q$_\E" } @ops;
+        $OPS_RE[$priority] = $ops;
+    }
+}
+
+
+push @EXPORT_OK, 'preload_operators';
+sub preload_operators
+{
+    require Module::Load;
+    
+    while( my( $keyword, $module) = each %KNOWN_OPS_PLAIN )
+    {
+        Module::Load::load($module);
+        $LOADED_MODULES{$module} = time;
+    }
+    
+    return 1;
+}
+
+
+require DTL::Fast::Template;
+require DTL::Fast::Cache::Runtime;
+require DTL::Fast::Cache::Serialized;
+
+
 1;
 
 __END__ 
@@ -215,7 +345,7 @@ DTL::Fast - Perl implementation of Django templating language.
 
 =head1 VERSION
 
-Version v1.05.6
+Version 1.609.0_1
 
 =head1 SYNOPSIS
 
@@ -239,7 +369,7 @@ And load and render it:
 
 This module is a Perl and stand-alone templating system, cloned from Django templating sytem, described in L<here|https://docs.djangoproject.com/en/1.7/topics/templates/>.
 
-=head2 GOALS
+=head2 Goals
 
 Goals of this implementation are:
 
@@ -253,15 +383,15 @@ Goals of this implementation are:
 
 =back
 
-=head2 CURRENT STATUS
+=head2 Current status
 
 Current release implements almost all tags and filters documented on Django site.
 
-There are no speed optimizations done yet.
+There are no significant speed optimizations done yet.
 
-Internationalization, localization and caching are not yet implemented.
+Internationalization and localization are not yet implemented.
 
-=head1 PERL SIDE
+=head1 BASICS
 
 You may get template object using three ways. 
 
@@ -275,7 +405,7 @@ Using DTL::Fast::Template constructor:
     my $tpl = DTL::Fast::Template->new(
         $template_text,                             # template itself
         'dirs' => [ $dir1, $dir2, ... ],            # optional, directories list to look for parent templates and includes
-        'ssi_dirs' => [ $ssi_dir1, $ssi_dir1, ...]  # optional, directories list to look for files included with ssi tag
+        'ssi_dirs' => [ $ssi_dir1, $ssi_dir1, ...]  # optional, directories list allowed to used in ssi tag (ALLOWED_INCLUDE_ROOTS in Django)
         'url_source' => \&uri_getter                # optional, reference to a function, that can return url template by model name (necessary for url tag)
     );
 
@@ -286,7 +416,7 @@ Using DTL::Fast::Template constructor:
     my $tpl = get_template(
         $template_path,                             # path to the template, relative to directories from second argument
         'dirs' => [ $dir1, $dir2, ... ],            # mandatory, directories list to look for parent templates and includes
-        'ssi_dirs' => [ $ssi_dir1, $ssi_dir1, ...]  # optional, directories list to look for files included with ssi tag
+        'ssi_dirs' => [ $ssi_dir1, $ssi_dir1, ...]  # optional, directories list allowed to used in ssi tag (ALLOWED_INCLUDE_ROOTS in Django)
         'url_source' => \&uri_getter                # optional, reference to a function, that can return url template by model name (necessary for url tag)
     );
     
@@ -299,7 +429,7 @@ when you are using C<get_template> helper function, framework will try to find t
     my $tpl = select_template(
         [ $template_path1, $template_path2, ...],   # paths to templates, relative to directories from second argument
         'dirs' => [ $dir1, $dir2, ... ],            # mandatory, directories list to look for parent templates and includes
-        'ssi_dirs' => [ $ssi_dir1, $ssi_dir1, ...]  # optional, directories list to look for files included with ssi tag
+        'ssi_dirs' => [ $ssi_dir1, $ssi_dir1, ...]  # optional, directories list allowed to used in ssi tag (ALLOWED_INCLUDE_ROOTS in Django)
         'url_source' => \&uri_getter                # optional, reference to a function, that can return url template by model name (necessary for url tag)
     );
     
@@ -340,15 +470,90 @@ or
     $context->set('name' => 'Sergey');
     print $tpl->render($context);
 
-=head2 Cache classes
+=head2 register_tag
+
+    use DTL::Fast qw(register_tag);
+    
+    register_tag(
+        'mytag' => 'MyTag::Module'
+    );
+    
+This method registers or overrides registered tag keyword with handler module. Module will be loaded when first encountered during template parsing. About handler modules you may read in L</CUSTOM TAGS> section.
+
+=head2 preload_tags
+
+    use DTL::Fast qw(preload_tags);
+    
+    preload_tags();
+    
+Preloads all registered tags modules. Mostly for debugging purposes or persistent environment stability.
+
+=head2 register_filter
+
+    use DTL::Fast qw(register_filter);
+    
+    register_filter(
+        'myfilter' => 'MyFilter::Module'
+    );
+    
+This method registers or overrides registered filter keyword with handler module. Module will be loaded when first encountered during template parsing. About handler modules you may read in L</CUSTOM FILTERS> section.
+
+=head2 preload_filters
+
+    use DTL::Fast qw(preload_filters);
+    
+    preload_filters();
+    
+Preloads all registered filters modules. Mostly for debugging purposes or persistent environment stability.
+
+=head2 register_operator
+
+    use DTL::Fast qw(register_operator);
+    
+    register_operator(
+        'xor' => [ 1, 'MyOps::XOR' ],
+        'myop' => [ 0, 'MyOps::MYOP' ],
+    );
+
+This method registers or overrides registered operator handlers. Handler module will be loaded when first encountered during template parsing. 
+
+Arguments hash is:
+
+    'operator_keyword' => [ precedence, handler_module ]
+
+Currently there are 9 precedences from 0 to 8, the lower is less prioritised. You may see built-in precedence in the C<DTL::Fast::Expression::Operator> module.
+
+More about custom operators you may read in L</CUSTOM OPERATORS> section.
+
+=head2 preload_filters
+
+    use DTL::Fast qw(preload_filters);
+    
+    preload_filters();
+    
+Preloads all registered filters modules. Mostly for debugging purposes or persistent environment stability.
+
+=head2 preload_operators
+
+    use DTL::Fast qw(preload_operators);
+    
+    preload_operators();
+    
+Preloads all registered operators modules. Mostly for debugging purposes or persistent environment stability.
+
+=head1 Cache classes
 
 To do...
 
-=head2 Custom tags
+=head1 CUSTOM TAGS
 
 To do...
 
-=head2 Custom filters
+=head1 CUSTOM FILTERS
+
+To do...
+
+=head1 CUSTOM OPERATORS
 
 To do...
 
@@ -367,15 +572,53 @@ New tag, that works like C<firstof> tag, but checks if value is defined (not tru
 
 C<url> tag works a different way. Because there is no framework around, we can't obtain model's path the same way. But you may pass C<url_source> parameter into template constructor or C<get_template>/C<select_template> function. This parameter MUST be a reference to a function, that will return to templating engine url template by some 'model path' (first parameter of C<url> tag). Second parameter passed to the C<url_source> handler will be a reference to array of argument values (in case of positional arguments) or reference to a hash of arguments (in case of named ones). Url source handler may just return a regexp template by model path and templating engine will try to restore it with specified arguments. Or, you may restore it yourself, alter replacement arguments or do whatever you want. 
 
+=head3 warn
+
+    {% warn var1 var2 ... varn %}
+
+C<warn> tag is useful for development and debugging. Dumps variable using L<C<Data::Dumper>> to C<STDERR>.
+
+Without and argument dumps full context object.
+
 =head2 Filters
 
 This module supports all built-in filters documented on L<official Django site|https://docs.djangoproject.com/en/1.7/ref/templates/builtins/#built-in-filter-reference>. Don't forget to read L<incompatibilities|/INCOMPATIBILITIES WITH DJANGO TEMPLATES> and L<extensions|/EXTENSIONS OF DJANGO TEMPLATES> sections.
 
+=head3 numberformat
+
+    {% var1|numberformat %}
+    
+Formats 12345678.9012 as
+    
+    12 345 678.9012
+
+Split integer part of the number by 3 digits, separated by spaces.
+
+=head3 reverse
+
+Reverses data depending on type:
+
+=over
+
+=item * Scalar will be reversed literally: "hi there" => "ereht ih"
+
+=item * Array will be reversed using perl's reverse function
+
+=item * Hash will be reversed using perl's reverse function
+
+=item * Object may provide reverse method to be used with this filter
+
+=back
+
+=head3 strftime
+
+Formatting timestamp using L<C<Date::Format>> module. This is C-style date formatting, not PHP one.
+
 =head1 INCOMPATIBILITIES WITH DJANGO TEMPLATES
 
 =over
 
-=item * C<ssi> tag in Django uses absolute paths and C<ALLOWED_INCLUDE_ROOTS> configuration option. This library works separately and may be used with different frameworks. So, C<ssi> tag uses relative paths and you MUST specify additional template constructor parameter: C<ssi_dirs> which should be an array reference with list of dirs to search in.
+=item * Django's setting C<ALLOWED_INCLUDE_ROOTS> should be passed to tempalte constructor/getter as C<ssi_dirs> argument.
 
 =item * C<csrf_token> tag is not implemented, too well connected with Django.
 
@@ -405,6 +648,8 @@ May be some of this features implemented in Django itself. Let me know about it.
 
 =over
 
+=item * filters may accept several arguments, and context variables can be used in them, like {{ var|filter1:var2:var3:...:varn }}
+
 =item * C<numberformat> - new filter. Formats number like C<12 345 678.9999999> 
 
 =item * C<strftime> - new filter. Formats time using L<C<Date::Format>> module, which is using C functions C<strftime> and C<ctime>.
@@ -486,10 +731,10 @@ Template parsing permormance with software cache wiping on each iteration:
 
     Benchmark: timing 5000 iterations of DTL::Fast  , Dotiac::DTL...
     
-    DTL::Fast  :  2 wallclock secs ( 1.31 usr +  0.75 sys =  2.06 CPU) @ 2428.36/s (n=5000)
-    Dotiac::DTL: 26 wallclock secs (11.40 usr + 13.01 sys = 24.41 CPU) @ 204.80/s (n=5000)
+    DTL::Fast  :  3 wallclock secs ( 2.47 usr +  1.05 sys =  3.51 CPU) @ 1424.10/s (n=5000)
+    Dotiac::DTL: 44 wallclock secs (21.86 usr + 20.39 sys = 42.25 CPU) @ 118.36/s (n=5000)
 
-C<DTL::Fast> parsing templates almost 12 times faster, than L<C<Dotiac::DTL>>.
+C<DTL::Fast> parsing templates 12 times faster, than L<C<Dotiac::DTL>>.
 
 To run this test, you need to alter L<C<Dotiac::DTL>> module and change declaration of C<my %cache;> to C<our %cache;>. 
     
@@ -497,21 +742,21 @@ Rendering of pre-compiled template (software cache):
 
     Benchmark: timing 3000 iterations of DTL::Fast  , Dotiac::DTL...
     
-    DTL::Fast  :  7 wallclock secs ( 7.29 usr +  0.00 sys =  7.29 CPU) @ 411.81/s (n=3000)
-    Dotiac::DTL: 13 wallclock secs (12.15 usr +  0.00 sys = 12.15 CPU) @ 246.85/s (n=3000)
+    DTL::Fast  : 17 wallclock secs (13.46 usr +  3.87 sys = 17.33 CPU) @ 173.09/s (n=3000)
+    Dotiac::DTL: 18 wallclock secs (17.38 usr +  0.00 sys = 17.38 CPU) @ 172.63/s (n=3000)
 
-Tests shows, that C<DTL::Fast> works 67% faster, than L<C<Dotiac::DTL>> in persistent environment.
+Tests shows, that C<DTL::Fast> works a bit faster, than L<C<Dotiac::DTL>> in persistent environment.
 
 =head2 CGI
 
 This test rendered test template many times by external script, invoked via C<system> call:
 
-    Benchmark: timing 500 iterations of Dotiac render , Fast render   ...
+    Benchmark: timing 300 iterations of Dotiac render     , Fast cached render, Fast render
     
-    Dotiac render : 57 wallclock secs ( 0.16 usr +  0.44 sys =  0.59 CPU) @ 843.17/s (n=500)
-    Fast render   : 62 wallclock secs ( 0.22 usr +  0.53 sys =  0.75 CPU) @ 668.45/s (n=500)
+    Dotiac render     : 40 wallclock secs ( 0.11 usr +  0.40 sys =  0.51 CPU) @ 583.66/s (n=300)
+    Fast render       : 55 wallclock secs ( 0.06 usr +  0.42 sys =  0.48 CPU) @ 619.83/s (n=300)
 
-Tests shows, that C<DTL::Fast> works 26% slower, than L<C<Dotiac::DTL>> in CGI environment.
+Tests shows, that C<DTL::Fast> works 40% slower, than L<C<Dotiac::DTL>> in CGI environment.
 
 =head2 DTL::Fast steps performance
 
@@ -525,27 +770,30 @@ Tests shows, that C<DTL::Fast> works 26% slower, than L<C<Dotiac::DTL>> in CGI e
     7 Parse      :  1 wallclock secs ( 0.44 usr +  0.23 sys =  0.67 CPU) @ 1492.54/s (n=1000)
     8 Render     : 11 wallclock secs ( 9.30 usr +  1.14 sys = 10.44 CPU) @ 95.82/s (n=1000)    
 
-
-=head1 BUGS AND IMPROVEMENTS
-
-If you found any bug and/or want to make some improvement, feel free to participate in the project on GitHub: L<https://github.com/hurricup/DTL-Fast>
-
-=head1 LICENSE
-
-This module is published under the terms of the MIT license, which basically means "Do with it whatever you want". For more information, see the LICENSE file that should be enclosed with this distributions. A copy of the license is (at the time of writing) also available at L<http://www.opensource.org/licenses/mit-license.php>.
-
 =head1 SEE ALSO
 
 =over
 
 =item * Main project repository and bugtracker: L<https://github.com/hurricup/DTL-Fast>
 
+=item * CPAN Testers reports: L<http://www.cpantesters.org/distro/D/DTL-Fast.html>
+
+=item * Testers matrix: L<http://matrix.cpantesters.org/?dist=DTL-Fast>
+        
+=item * AnnoCPAN, Annotated CPAN documentation: L<http://annocpan.org/dist/DTL-Fast>
+
+=item * CPAN Ratings: L<http://cpanratings.perl.org/d/DTL-Fast>
+
 =item * Original Django templating documentation: L<https://docs.djangoproject.com/en/1.7/topics/templates/>
 
 =item * Other implementaion: L<http://search.cpan.org/~maluku/Dotiac-0.8/lib/Dotiac/DTL.pm>
 
 =back
 
+=head1 LICENSE
+
+This module is published under the terms of the MIT license, which basically means "Do with it whatever you want". For more information, see the LICENSE file that should be enclosed with this distributions. A copy of the license is (at the time of writing) also available at L<http://www.opensource.org/licenses/mit-license.php>.
+
 =head1 AUTHOR
 
 Copyright (C) 2014-2015 by Alexandr Evstigneev (L<hurricup@evstigneev.com|mailto:hurricup@evstigneev.com>)
