@@ -1,19 +1,13 @@
 package DTL::Fast::Expression::Operator::Unary;
 use strict; use utf8; use warnings FATAL => 'all'; 
-use Carp;
 
-use DTL::Fast::Expression::Operator::Unary::Not;
-
-# not in Django
-use DTL::Fast::Expression::Operator::Unary::Defined;
+use DTL::Fast::Template;
 
 sub new
 {
     my( $proto, $argument, %kwargs ) = @_;
     $kwargs{'a'} = $argument;
-    $kwargs{'_template'}->{'modules'}->{$proto} //= $proto->VERSION // $DTL::Fast::VERSION;
-
-    delete $kwargs{'_template'};
+    $DTL::Fast::Template::CURRENT_TEMPLATE->{'modules'}->{$proto} //= $proto->VERSION // DTL::Fast->VERSION;
     
     return bless {%kwargs}, $proto;
 }
@@ -31,6 +25,6 @@ sub render
 sub dispatch
 {
     my( $self, $arg1 ) = @_;
-    croak 'ABSTRACT: This method should be overriden in subclasses';
+    die 'ABSTRACT: This method should be overriden in subclasses';
 }
 1;
\ No newline at end of file
