@@ -1,15 +1,13 @@
 package DTL::Fast::Template;
 use strict; use utf8; use warnings FATAL => 'all'; 
 use parent 'DTL::Fast::Parser';
-use Carp;
-
-our %TAG_HANDLERS;
-our %FILTER_HANDLERS;
 
 use DTL::Fast::Context;
 use DTL::Fast::Tags;
 use DTL::Fast::Filters;    
 
+our $CURRENT_TEMPLATE;  # global variable for linking  modules
+
 #@Override
 sub new
 {
@@ -28,7 +26,7 @@ sub new
 
     if( exists $kwargs{'inherits'}->{$kwargs{'file_path'}} )
     {
-        croak  sprintf(
+        die  sprintf(
             "Recursive inheritance detected:\n%s\n" 
             , join "\n inherited from ", @{$kwargs{'inherited'}}, $kwargs{'file_path'}
         );
@@ -41,7 +39,7 @@ sub new
     push @{$kwargs{'inherited'}}, $kwargs{'file_path'};
     
     my $self = $proto->SUPER::new(%kwargs);
-
+    
     if( $self->{'extends'} )
     {
         my $parent_template = DTL::Fast::read_template(
@@ -55,7 +53,7 @@ sub new
         }
         else
         {
-            croak  sprintf( "Couldn't found a parent template: %s in one of the following directories: %s"
+            die  sprintf( "Couldn't found a parent template: %s in one of the following directories: %s"
                 , $self->{'extends' }
                 , join( ', ', @{$kwargs{'dirs'}})
             );
@@ -87,19 +85,34 @@ sub merge_template
     return $self;
 }
 
+#@Override
+sub parse_chunks
+{
+    my( $self ) = @_;
+    
+    my $current_template_backup = $CURRENT_TEMPLATE;
+    $CURRENT_TEMPLATE = $self;
+    
+    $self->SUPER::parse_chunks();
+    
+    $CURRENT_TEMPLATE = $current_template_backup;
+    return $self;
+}
+
 #@Override
 sub get_container_block{ return shift; }
 
+my $reg = qr/(
+     \{\#.+?\#\}
+    |\{\%.+?\%\}
+    |\{\{.+?\}\}
+)/xs;
+
 sub _get_raw_chunks
 {
     my( $template ) = @_;
 
-    my $reg = qr/(
-        \{\# .*? \#\}
-        |\{\% .+? \%\}
-        |\{\{ .+? \}\}
-    )/x;
-    my $result = [split /$reg/s, $template];
+    my $result = [split $reg, $template];
     
     return $result;    
 }
@@ -120,7 +133,7 @@ sub render
         and ref $context ne 'DTL::Fast::Context'
     )
     {
-        croak  "Context must be a DTL::Fast::Context object or a HASH reference";
+        die  "Context must be a DTL::Fast::Context object or a HASH reference";
     }
     
     $context->push_scope();
